const dashboard = {
    state () {
        return {
            Budgets: [],
            CurrentBudget: null,
            Accounts: [],
            CurrentAccount: null,
            Categories: [],
            Transactions: [],
            Assignments: []
        }
    },
    mutations: {
        setBudgets (state, budgets) {
            state.Budgets = budgets;
        },
        addBudget(state, budget) {
            state.Budgets.push(budget);
        },
        setCurrentBudget(state, budget) {
            state.CurrentBudget = budget;
        },
        setAccounts (state, accounts) {
            state.Accounts = accounts;
        },
        setCurrentAccount(state, account) {
            state.CurrentAccount = account;
        },
        setTransactions(state, transactions){
            state.Transactions = transactions;
        }
    },
    getters: {
        Budgets(state) {
            return state.Budgets || [];
        },
        CurrentBudget(state) {
            return state.CurrentBudget || {};
        },
        Accounts(state) {
            return state.Accounts || [];
        },
        CurrentAccount(state) {
            return state.CurrentAccount || {};
        },
        OnBudgetAccounts(state) {
            return (state.Accounts || []).filter(x => x.OnBudget);
        },
        OffBudgetAccounts(state) {
            return (state.Accounts || []).filter(x => !x.OnBudget);
        },
        Transactions(state) {
            return (state.Transactions || []);
        }
    },
    actions: {
        async fetchDashboard ({state, commit, rootState}) {
            const response = await fetch("/api/v1/dashboard", {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            })
            const data = await response.json();
            commit("setBudgets", data.Budgets);
        },
        async newBudget ({state, commit, rootState}, budgetName) {
            fetch("/api/v1/budget/new", {
                method: "POST",
                body: JSON.stringify({name: budgetName}),
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            })
                .then(x => x.json())
                .then(x => commit("addBudget", x));
        },
        async setCurrentBudget({state, commit, dispatch}, budgetid) {
            await dispatch("fetchDashboard");
            for (const element of state.Budgets) {
                if(element.ID != budgetid) 
                    continue
                
                commit("setCurrentBudget", element);
                break
            }
        },
        async fetchBudget ({state, commit, rootState}, budgetid) {
            const result = await fetch("/api/v1/budget/" + budgetid, {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            });
            const response = await result.json();
            return commit("setAccounts", response.Accounts);
        },
        async fetchAccount ({state, commit, rootState}, accountid) {
            const result = await fetch("/api/v1/account/" + accountid + "/transactions", {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            });
            const response = await result.json();
            return commit("setTransactions", response.Transactions);
        },
        async setCurrentAccount({state, commit, dispatch}, {budgetid, accountid}) {
            await dispatch("fetchBudget", budgetid);
            for (const element of state.Accounts) {
                if(element.ID != accountid) 
                    continue
                
                commit("setCurrentAccount", element);
                commit(TITLE, element.Name);
                break
            }
        }
    }
}

export default dashboard